@@ -1,6 +1,6 @@
 /*
  *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 161 $, $Date: 2007-05-05 22:53:24 +0100 (Sat, 05 May 2007) $
  *  Copyright 2007 RW Penney
  */
 
@@ -171,6 +171,7 @@ static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
 #endif
 
     /* read key header: */
+    hbuff[0] = '\0';
     (void)fread((void*)hbuff, (size_t)1, kmblti_maglen, fp_key);
     if (strncmp((const char*)hbuff, kmblti_magstr, kmblti_maglen) != 0) {
         fprintf(stderr, "bad keyfile format (builtin)\n");
@@ -210,14 +211,14 @@ static int kmblti_get_key(const char *ident, const keyinfo_t *keyinfo,
         iv[0] = cv[0];
         iv[1] = cv[1];
 
-        bptr[7] = (pl[0] & 0xff000000) >> 24;
-        bptr[6] = (pl[0] & 0xff0000) >> 16;
-        bptr[5] = (pl[0] & 0xff00) >> 8;
-        bptr[4] = (pl[0] & 0xff);
-        bptr[3] = (pl[1] & 0xff000000) >> 24;
-        bptr[2] = (pl[1] & 0xff0000) >> 16;
-        bptr[1] = (pl[1] & 0xff00) >> 8;
-        bptr[0] = (pl[1] & 0xff);
+        bptr[7] = (unsigned char)((pl[0] >> 24) & 0xff);
+        bptr[6] = (unsigned char)((pl[0] >> 16) & 0xff);
+        bptr[5] = (unsigned char)((pl[0] >> 8) & 0xff);
+        bptr[4] = (unsigned char)(pl[0] & 0xff);
+        bptr[3] = (unsigned char)((pl[1] >> 24) & 0xff);
+        bptr[2] = (unsigned char)((pl[1] >> 16) & 0xff);
+        bptr[1] = (unsigned char)((pl[1] >> 8) & 0xff);
+        bptr[0] = (unsigned char)(pl[1] & 0xff);
 
         bptr += 8;
     }
@@ -299,14 +300,14 @@ static int kmblti_put_key(const char *ident, const keyinfo_t *keyinfo,
         iv[0] = cv[0];
         iv[1] = cv[1];
 
-        bptr[7] = (cv[0] & 0xff000000) >> 24;
-        bptr[6] = (cv[0] & 0xff0000) >> 16;
-        bptr[5] = (cv[0] & 0xff00) >> 8;
-        bptr[4] = (cv[0] & 0xff);
-        bptr[3] = (cv[1] & 0xff000000) >> 24;
-        bptr[2] = (cv[1] & 0xff0000) >> 16;
-        bptr[1] = (cv[1] & 0xff00) >> 8;
-        bptr[0] = (cv[1] & 0xff);
+        bptr[7] = (unsigned char)((cv[0] >> 24) & 0xff);
+        bptr[6] = (unsigned char)((cv[0] >> 16) & 0xff);
+        bptr[5] = (unsigned char)((cv[0] >> 8) & 0xff);
+        bptr[4] = (unsigned char)(cv[0] & 0xff);
+        bptr[3] = (unsigned char)((cv[1] >> 24) & 0xff);
+        bptr[2] = (unsigned char)((cv[1] >> 16) & 0xff);
+        bptr[1] = (unsigned char)((cv[1] >> 8) & 0xff);
+        bptr[0] = (unsigned char)(cv[1] & 0xff);
 
         bptr += 8;
     }
