@@ -1,6 +1,6 @@
 /*
  *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 171 $, $Date: 2007-07-17 20:55:35 +0100 (Tue, 17 Jul 2007) $
  *  Copyright 2005-2007 RW Penney
  */
 
@@ -221,16 +221,20 @@ static keymanager_t keymgr_raw = {
 #ifndef AS_MODULE
 
 static keymanager_t **add_keymgr(keymanager_t **ptr, keymanager_t *km)
-    /* add key-manager interface definition to list of available managers */
+    /* add key-manager interface definition(s) to list of available managers */
 {
     if (km == NULL) return ptr;
 
     *ptr = km;
-    km->next = NULL;
 
+    /* allow for addend itself being a list of key-managers: */
+    for (;;) {
 #ifdef TESTING
-    if (km->install_testctxt != NULL) km->install_testctxt(test_ctxtptr);
+        if (km->install_testctxt != NULL) km->install_testctxt(test_ctxtptr);
 #endif
+        if (km->next == NULL) break;
+        km = km->next;
+    }
 
     return &km->next;
 }
@@ -614,7 +618,7 @@ int cm_mutex_lock(void)
             if (errno == EEXIST) {
                 /* try again later */
 #if HAVE_NANOSLEEP
-                ticks = (53 * tries + 97 * (int)&tries) % 11;
+                ticks = (53 * tries + 97 * (long)&tries) % 11;
                 delay.tv_sec = (ticks / 10);
                 delay.tv_nsec = (ticks % 10) * 1000L * 1000L * 1000L;
                 nanosleep(&delay, NULL);
