@@ -1,6 +1,6 @@
 /*
  *  declarations for unit-test utilities for cryptmoumt
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 161 $, $Date: 2007-05-05 22:53:24 +0100 (Sat, 05 May 2007) $
  *  Copyright 2006-2007, RW Penney
  */
 
@@ -66,7 +66,8 @@ extern cm_testinfo_t *test_ctxtptr;
 
 #define CM_ASSERT_EQUAL(expected, actual) \
     if ((expected) != (actual)) { \
-        fprintf(stderr, "  test failed %s:%d\n", __FILE__, __LINE__); \
+        fprintf(stderr, "  test failed %s:%d  (%s != %s)\n", \
+                 __FILE__, __LINE__, #expected, #actual); \
         if (test_ctxtptr->current != NULL) { \
             fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
         ++test_ctxtptr->test_stats[CM_TEST_FAILED]; \
@@ -75,7 +76,8 @@ extern cm_testinfo_t *test_ctxtptr;
 
 #define CM_ASSERT_DIFFERENT(expected, actual) \
     if ((expected) == (actual)) { \
-        fprintf(stderr, "  test failed %s:%d\n", __FILE__, __LINE__); \
+        fprintf(stderr, "  test failed %s:%d\n  (%s == %s)", \
+                 __FILE__, __LINE__, #expected, #actual); \
         if (test_ctxtptr->current != NULL) { \
             fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
         ++test_ctxtptr->test_stats[CM_TEST_FAILED]; \
