@@ -1,6 +1,6 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 170 $, $Date: 2007-07-15 17:53:26 +0100 (Sun, 15 Jul 2007) $
  *  Copyright 2005-2007 RW Penney
  */
 
@@ -42,6 +42,7 @@
 #include <sys/mount.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #ifdef HAVE_SYSLOG
 #  include <syslog.h>
 #endif
@@ -56,6 +57,9 @@
 #include "looputils.h"
 #include "tables.h"
 #include "utils.h"
+#if WITH_CSWAP
+#  include <sys/swap.h>
+#endif
 #ifdef TESTING
 #  include "cmtesting.h"
 #endif
@@ -76,13 +80,14 @@ typedef enum
     M_MOUNT, M_UNMOUNT,
     M_SWAPON, M_SWAPOFF,
     M_LIST, M_KEYMGRS,
-    M_PASSWORD, M_KEYGEN, M_VERSION
+    M_PASSWORD, M_KEYGEN, M_KEYREU,
+    M_SAFETYNET, M_VERSION
 } cmmode_t;
 
 
 static int64_t getblkcount(const char *device);
-static int execute_list(cmmode_t mode, const char *params,
-                const targelt_t *eltlist);
+static int execute_list(cmmode_t mode, const cment_t *cmtable,
+                const char *params, const targelt_t *eltlist);
 static int do_list(const cment_t *cment);
 static int do_devsetup(const cment_t *cment, char **mntdev);
 static int do_devshutdown(const cment_t *cment);
@@ -91,7 +96,9 @@ static int do_unmount(const cment_t *cment);
 static int do_swapon(const cment_t *cment);
 static int do_swapoff(const cment_t *cment);
 static int do_passwd(const cment_t *cment);
-static int do_keygen(const cment_t *cment, const char *params);
+static int do_keygen(const cment_t *cment, const char *params,
+                int reuse, const cment_t *cmtable);
+static int do_safetynet();
 
 
 static const char *USAGE_STRING = N_("\
@@ -107,6 +114,7 @@ usage: cryptmount [OPTION [target ...]]\n\
     -m | --mount <target>\n\
     -u | --unmount <target>\n\
     --generate-key <key-size> <target>\n\
+    --reuse-key <src-target> <dst-target>\n\
     --prepare <target>\n\
     --release <target>\n\
     --config-fd <num>\n\
@@ -127,7 +135,7 @@ int fs_test_blkgetsz()
     /* check that 32bit & 64bit device size calculations agree */
 {
 #ifdef BLKGETSIZE64
-    int fd, n_open, seclen, s0, s1, s2;
+    int fd, n_open, seclen;
     long len;
     uint64_t len64;
     const char **dev;
@@ -158,10 +166,10 @@ int fs_test_blkgetsz()
             }
             close(fd);
             if (len64 < (1<<31)) {
-                CM_ASSERT_EQUAL(len64, ((int64_t)len * (int64_t)seclen));
+                CM_ASSERT_EQUAL(len64, ((int64_t)len * (int64_t)512));
             }
-            /* fprintf(stderr, "%s: %d  %ld  %lld\n", *dev, seclen, len, len64 / seclen); */
         }
+        /* fprintf(stderr, "%s: %d  %ld  %lld\n", *dev, seclen, len * 512, len64); */
         ++dev;
     }
 
@@ -198,6 +206,8 @@ static int64_t getblkcount(const char *device)
     }
 #else
     if (ioctl(fd, BLKGETSIZE, &len) == 0) {
+        /*  this is the number of 512-byte blocks,
+            which may be the best we can do without BLKGETSIZE64... */
         count = (int64_t)len;
     }
 #endif
@@ -328,10 +338,9 @@ static int do_devsetup(const cment_t *cment, char **mntdev)
 int do_devshutdown(const cment_t *cment)
     /* remove all devices attached to encrypted target */
 {   struct stat sbuff;
-    unsigned i,devcnt=0;
+    unsigned devcnt=0;
     int eflag=ERR_NOERROR;
     dev_t *devids=NULL;
-    char buff[256];
 
     /* check if filing system has been configured at all: */
     if (!is_configured(cment->ident, NULL)) {
@@ -364,12 +373,7 @@ int do_devshutdown(const cment_t *cment)
 
     /* tidy-up any associated loopback devices: */
     if (S_ISREG(sbuff.st_mode) && devids != NULL) {
-        for (i=0; i<devcnt; ++i) {
-            if (loop_ident(major(devids[i]),minor(devids[i]),buff,sizeof(buff)) || (loop_destroy(buff) != ERR_NOERROR)) {
-                fprintf(stderr, _("failed to free device (%d,%d)\n"),
-                        major(devids[i]), minor(devids[i]));
-            }
-        }
+        (void)loop_dellist(devcnt, devids);
     }
 
   bail_out:
@@ -591,15 +595,22 @@ static int do_passwd(const cment_t *cment)
     if (fclose(fp) != 0) eflag = ERR_BADFILE;
 
     if (eflag == ERR_NOERROR) {
-        /* FIXME - check return codes */
-        (void)stat(cment->key.filename, &sbuff);
-        (void)rename(cment->key.filename, oldfname);
-        (void)chown(oldfname, 0, 0);
-        (void)chmod(oldfname, S_IRUSR | S_IWUSR);
+        if (stat(cment->key.filename, &sbuff) != 0
+          || rename(cment->key.filename, oldfname) != 0
+          || chown(oldfname, 0, 0) != 0
+          || chmod(oldfname, S_IRUSR | S_IWUSR) != 0) {
+            fprintf(stderr, _("retiring old key (%s -> %s) failed)"),
+                    cment->key.filename, oldfname);
+            goto bail_out;
+        }
 
-        (void)rename(newfname, cment->key.filename);
-        (void)chown(cment->key.filename, sbuff.st_uid, sbuff.st_gid);
-        (void)chmod(cment->key.filename, sbuff.st_mode);
+        if (rename(newfname, cment->key.filename) != 0
+          || chown(cment->key.filename, sbuff.st_uid, sbuff.st_gid) != 0
+          || chmod(cment->key.filename, sbuff.st_mode) != 0) {
+            fprintf(stderr, _("installing new key (%s -> %s) failed"),
+                    newfname, cment->key.filename);
+            goto bail_out;
+        }
 
         newfname = NULL;
 
@@ -617,20 +628,35 @@ static int do_passwd(const cment_t *cment)
 }
 
 
-static int do_keygen(const cment_t *cment, const char *params)
+static int do_keygen(const cment_t *cment, const char *params,
+                    int reuse, const cment_t *cmtable)
+    /* create new filesystem crypto-key */
 {   unsigned char *key=NULL;
     int keylen=0, eflag=ERR_NOERROR;
     char *fname=NULL;
+    const cment_t *parent=NULL;
     size_t sz;
     struct stat sbuff;
     FILE *fp;
 
-    if (params == NULL || sscanf(params, "%i", &keylen) != 1 || keylen < 0) {
-        fprintf(stderr, _("bad key-length parameter"));
-        eflag = ERR_BADPARAM;
-        goto bail_out;
+    if (params != NULL) {
+        if (reuse) {
+            parent = get_cment(cmtable, params);
+            if (parent == NULL) {
+                fprintf(stderr, _("target name \"%s\" is not recognized\n"),
+                        params);
+                eflag = ERR_BADPARAM;
+            }
+        } else {
+            if (sscanf(params, "%i", &keylen) != 1 || keylen < 1) {
+                fprintf(stderr, _("bad key-length parameter"));
+                eflag = ERR_BADPARAM;
+            }
+        }
     }
+    if (params == NULL || eflag != ERR_NOERROR) goto bail_out;
 
+    /* check if keyfile already exists: */
     if (stat(cment->key.filename, &sbuff) == 0) {
         fprintf(stderr,_("key-file \"%s\" already exists for target \"%s\"\n"),
             cment->key.filename, cment->ident);
@@ -638,12 +664,16 @@ static int do_keygen(const cment_t *cment, const char *params)
         goto bail_out;
     }
 
-    fprintf(stderr, _("generating random key; please be patient...\n"));
-    key = (unsigned char*)sec_realloc(NULL, (size_t)keylen);
-    eflag = get_randkey(key, (size_t)keylen);
-    if (eflag != ERR_NOERROR) {
-        fprintf(stderr, _("failed to generate new key\n"));
-        goto bail_out;
+    if (reuse) {
+        eflag = get_key(parent->ident, &parent->key, &key, &keylen);
+    } else {
+        fprintf(stderr, _("generating random key; please be patient...\n"));
+        key = (unsigned char*)sec_realloc(NULL, (size_t)keylen);
+        eflag = get_randkey(key, (size_t)keylen);
+        if (eflag != ERR_NOERROR) {
+            fprintf(stderr, _("failed to generate new key\n"));
+            goto bail_out;
+        }
     }
 
     sz = strlen(cment->key.filename) + 16;
@@ -660,10 +690,13 @@ static int do_keygen(const cment_t *cment, const char *params)
     fclose(fp);
 
     if (eflag == ERR_NOERROR) {
-        /* FIXME - check return codes */
-        (void)rename(fname, cment->key.filename);
-        (void)chown(cment->key.filename, 0, 0);
-        (void)chmod(cment->key.filename, S_IRUSR | S_IWUSR);
+        if (rename(fname, cment->key.filename) != 0
+          || chown(cment->key.filename, 0, 0) != 0
+          || chmod(cment->key.filename, S_IRUSR | S_IWUSR) != 0) {
+            fprintf(stderr, _("installation of new keyfile \"%s\" failed"),
+                    cment->key.filename);
+            eflag = ERR_BADFILE;
+        }
     }
 
   bail_out:
@@ -675,6 +708,91 @@ static int do_keygen(const cment_t *cment, const char *params)
 }
 
 
+static int do_safetynet()
+    /* last-ditch attempt to umount/shutdown all targets currently mounted */
+{   tgtstat_t *tgts=NULL, *tgt;
+    char *devname=NULL, *old_ident=NULL;
+    cment_t *cment=NULL;
+    dev_t *devids=NULL;
+    unsigned devcnt=0;
+    int mflag;
+    struct {
+        unsigned targets, unmounted, unswapped, undeviced, unlooped; }
+        counts = {
+            0, 0, 0, 0, 0 };
+
+    /*  This routine is ugly, but may be the best we can do to prevent
+     *  damage to filesystems that should have been properly removed
+     *  by other mechanisms (e.g. --unmount, --swapoff) */
+
+    cment = alloc_cment();
+    old_ident = cment->ident;
+
+    /* get list of all targets in status-file: */
+    tgts = get_all_tgtstatus();
+
+    for (tgt=tgts; tgt!=NULL; tgt=tgt->nx) {
+        ++counts.targets;
+        devmap_path(&devname, tgt->ident);
+        cment->ident = tgt->ident;
+
+#ifdef DLGT_UMOUNT
+        /* attempt to unmount filesystem: */
+        switch (fork()) {
+            case -1:
+                break;
+            case 0:
+                execl(DLGT_UMOUNT, "umount", devname, NULL);
+                break;
+            default:
+                (void)wait(&mflag);
+                break;
+        }
+        if (mflag == 0) ++counts.unmounted;
+#endif  /* DLGT_UMOUNT */
+#if WITH_CSWAP
+        /* attempt to remove swap partition: */
+        if (swapoff(devname) == 0) ++counts.unswapped;
+#endif  /* WITH_CSWAP */
+
+        /* remove device-mapper device: */
+        (void)devmap_dependencies(tgt->ident, &devcnt, &devids);
+        if (devmap_remove(tgt->ident) == ERR_NOERROR) ++counts.undeviced;
+
+        /* free any associated loopback devices: */
+        if (devcnt > 0 && loop_dellist(devcnt, devids) == 0) ++counts.unlooped;
+        if (devids != NULL) {
+            free((void*)devids);
+            devids = NULL;
+        }
+
+        /* remove entry in status-file, having done our best to clean-up: */
+        (void)put_tgtstatus(cment, NULL);
+
+        if (devname != NULL) {
+            free((void*)devname);
+            devname = NULL;
+        }
+    }
+
+    free_tgtstatus(tgts);
+
+    cment->ident = old_ident;
+    free_cment(cment);
+
+    if (counts.targets != 0) {
+        fprintf(stderr, "safety-net caught %u targets:\n"
+                "\t%u unmounted, %u swaps removed\n"
+                "\t%u devices removed, %u loopbacks freed\n",
+                counts.targets,
+                counts.unmounted, counts.unswapped,
+                counts.undeviced, counts.unlooped);
+    }
+
+    return (counts.targets != counts.undeviced);
+}
+
+
 static void check_priv_opt(const char *opt)
     /* check if ordinary user is allowed to perform privileged actions */
 {
@@ -705,9 +823,9 @@ static int check_mode(cmmode_t mode, const char *params,
                     const targelt_t *eltlist)
     /* check that arguments supplied & privilege match selected mode */
 {   enum {
-        F_NORMALUSER =  0x001,
-        F_NEEDS_TGT =   0x010,
-        F_NEEDS_PRM =   0x020
+        F_NORMALUSER =  0x001,      /* normal user can perform action */
+        F_NEEDS_TGT =   0x010,      /* filesystem-target is required */
+        F_NEEDS_PRM =   0x020       /* additional parameter is required */
     };
     struct tblent {
         cmmode_t mode;
@@ -723,7 +841,9 @@ static int check_mode(cmmode_t mode, const char *params,
         { M_LIST,       F_NORMALUSER },
         { M_KEYMGRS,    F_NORMALUSER },
         { M_PASSWORD,   F_NORMALUSER | F_NEEDS_TGT },
-        { M_KEYGEN,     F_NEEDS_TGT | F_NEEDS_PRM},
+        { M_KEYGEN,     F_NEEDS_TGT | F_NEEDS_PRM },
+        { M_KEYREU,     F_NEEDS_TGT | F_NEEDS_PRM },
+        { M_SAFETYNET,  0u },
         { M_VERSION,    F_NORMALUSER },
     };
     unsigned idx,nmds;
@@ -754,8 +874,8 @@ static int check_mode(cmmode_t mode, const char *params,
 }
 
 
-static int execute_list(cmmode_t mode, const char *params,
-                 const targelt_t *eltlist)
+static int execute_list(cmmode_t mode, const cment_t *cmtable,
+                const char *params, const targelt_t *eltlist)
 {   const targelt_t *elt;
     int i, prio=0, eflag=ERR_NOERROR;
     struct passwd *pwent=NULL;
@@ -771,9 +891,10 @@ static int execute_list(cmmode_t mode, const char *params,
     /* check that we have suitable privileges + parameters: */
     (void)check_mode(mode, params, eltlist);
 
-    /* execute user-selected task: */
+    /* execute special-cases of user-selected task: */
     if (mode == M_VERSION) {
         fprintf(stderr, "%s-%s\n", PACKAGE_NAME, PACKAGE_VERSION);
+        eltlist = NULL;
     } else if (mode == M_KEYMGRS) {
         keymgrs = get_keymgr_list();
         for (i=0; keymgrs[i]!=NULL; ) {
@@ -781,64 +902,74 @@ static int execute_list(cmmode_t mode, const char *params,
             if (keymgrs[++i] != NULL) printf(", "); else printf("\n");
         }
         free((void*)keymgrs);
-    } else {
-        for (elt=eltlist; elt!=NULL && eflag<ERR_threshold; elt=elt->nx) {
-            syslogmsg = NULL;
-            prio = LOG_AUTHPRIV | LOG_NOTICE;
-            switch (mode) {
-                case M_LIST:
-                    eflag = do_list(elt->entry);
-                    break;
-                case M_PREPARE:
-                    syslogmsg = "prepare of \"%s\" by %s %s";
-                    eflag = do_devsetup(elt->entry, NULL);
-                    break;
-                case M_RELEASE:
-                    syslogmsg = "release of \"%s\" by %s %s";
-                    prio = LOG_AUTHPRIV | LOG_NOTICE;
-                    eflag = do_devshutdown(elt->entry);
-                    if (eflag == WRN_UNCONFIG) syslogmsg = NULL;
-                    break;
-                case M_MOUNT:
-                    if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
-                    syslogmsg = "mount of \"%s\" by %s %s";
-                    eflag = do_mount(elt->entry);
-                    break;
-                case M_UNMOUNT:
-                    if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
-                    syslogmsg = "unmount of \"%s\" by %s %s";
-                    eflag = do_unmount(elt->entry);
-                    if (eflag == WRN_UNCONFIG) syslogmsg = NULL;
-                    break;
-                case M_SWAPON:
-                    syslogmsg = "swapon \"%s\" by %s %s";
-                    eflag = do_swapon(elt->entry);
-                    break;
-                case M_SWAPOFF:
-                    syslogmsg = "swapoff \"%s\" by %s %s";
-                    eflag = do_swapoff(elt->entry);
-                    if (eflag == WRN_UNCONFIG) syslogmsg = NULL;
-                    break;
-                case M_PASSWORD:
-                    if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
-                    syslogmsg = "changing password for \"%s\" by %s %s";
-                    eflag = do_passwd(elt->entry);
-                    break;
-                case M_KEYGEN:
-                    if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
-                    syslogmsg = "key generation for \"%s\" by %s %s";
-                    eflag = do_keygen(elt->entry, params);
-                    break;
-                default:
-                    break;
-            }
-#if defined(HAVE_SYSLOG) && !defined(TESTING)
-            if (syslogmsg != NULL) {
-                syslog(prio, syslogmsg, elt->entry->ident, username,
-                    (eflag == ERR_NOERROR ? "succeeded" : "failed"));
-            }
-#endif
+        eltlist = NULL;
+    } else if (mode == M_SAFETYNET) {
+        do_safetynet();
+        eltlist = NULL;
+    }
+
+    /* apply user-selected operation to list of targets (if present): */
+    for (elt=eltlist; elt!=NULL && eflag<ERR_threshold; elt=elt->nx) {
+        syslogmsg = NULL;
+        prio = LOG_AUTHPRIV | LOG_NOTICE;
+        switch (mode) {
+            case M_LIST:
+                eflag = do_list(elt->entry);
+                break;
+            case M_PREPARE:
+                syslogmsg = "prepare of \"%s\" by %s %s";
+                eflag = do_devsetup(elt->entry, NULL);
+                break;
+            case M_RELEASE:
+                syslogmsg = "release of \"%s\" by %s %s";
+                prio = LOG_AUTHPRIV | LOG_NOTICE;
+                eflag = do_devshutdown(elt->entry);
+                if (eflag == WRN_UNCONFIG) syslogmsg = NULL;
+                break;
+            case M_MOUNT:
+                if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
+                syslogmsg = "mount of \"%s\" by %s %s";
+                eflag = do_mount(elt->entry);
+                break;
+            case M_UNMOUNT:
+                if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
+                syslogmsg = "unmount of \"%s\" by %s %s";
+                eflag = do_unmount(elt->entry);
+                if (eflag == WRN_UNCONFIG) syslogmsg = NULL;
+                break;
+            case M_SWAPON:
+                syslogmsg = "swapon \"%s\" by %s %s";
+                eflag = do_swapon(elt->entry);
+                break;
+            case M_SWAPOFF:
+                syslogmsg = "swapoff \"%s\" by %s %s";
+                eflag = do_swapoff(elt->entry);
+                if (eflag == WRN_UNCONFIG) syslogmsg = NULL;
+                break;
+            case M_PASSWORD:
+                if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
+                syslogmsg = "changing password for \"%s\" by %s %s";
+                eflag = do_passwd(elt->entry);
+                break;
+            case M_KEYGEN:
+                if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
+                syslogmsg = "key generation for \"%s\" by %s %s";
+                eflag = do_keygen(elt->entry, params, 0, NULL);
+                break;
+            case M_KEYREU:
+                if ((eflag = check_priv_tgt(elt->entry)) != ERR_NOERROR) break;
+                syslogmsg = "key generation for \"%s\" by %s %s";
+                eflag = do_keygen(elt->entry, params, 1, cmtable);
+                break;
+            default:
+                break;
         }
+#if defined(HAVE_SYSLOG) && !defined(TESTING)
+        if (syslogmsg != NULL) {
+            syslog(prio, syslogmsg, elt->entry->ident, username,
+                (eflag == ERR_NOERROR ? "succeeded" : "failed"));
+        }
+#endif
     }
 
 #ifdef HAVE_SYSLOG
@@ -905,6 +1036,8 @@ int main(int argc, char *argv[])
         { "mount",              no_argument,        NULL, (int)'m' },
         { "prepare",            no_argument,        NULL, (int)'p' },
         { "release",            no_argument,        NULL, (int)'r' },
+        { "reuse-key",          required_argument,  NULL, (int)'e' },
+        { "safetynet",          no_argument,        NULL, (int)'n' },
         { "swapon",             no_argument,        NULL, (int)'s' },
         { "swapoff",            no_argument,        NULL, (int)'x' },
         { "unmount",            no_argument,        NULL, (int)'u' },
@@ -917,7 +1050,7 @@ int main(int argc, char *argv[])
 #  endif
         { NULL, 0, NULL, 0 } };
 #endif
-    const char *params=NULL, *optlist="acf:g:klmprsuvx";
+    const char *params=NULL, *optlist="acdf:g:klmprsuvx";
     char *cmtab=NULL;
     int val,idx,config_fd=-1,eflag=ERR_NOERROR;
     cment_t *cmtable=NULL;
@@ -953,6 +1086,8 @@ int main(int argc, char *argv[])
                 flags |= F_ALL; break;
             case 'c':
                 mode = M_PASSWORD; break;
+            case 'e':                       check_priv_opt("--reuse-key");
+                mode = M_KEYREU; params = optarg; break;
             case 'f':                       check_priv_opt("--config-fd");
                 sscanf(optarg, "%d", &config_fd); break;
             case 'g':                       check_priv_opt("--generate-key");
@@ -965,6 +1100,8 @@ int main(int argc, char *argv[])
                 mode = M_LIST;  break;
             case 'm':
                 mode = M_MOUNT; break;
+            case 'n':                       check_priv_opt("--safetynet");
+                mode = M_SAFETYNET; break;
             case 'p':                       check_priv_opt("--prepare");
                 mode = M_PREPARE; break;
             case 'r':                       check_priv_opt("--release");
@@ -1063,7 +1200,7 @@ int main(int argc, char *argv[])
 
 
     /* execute user-selected task: */
-    eflag = execute_list(mode, params, eltlist);
+    eflag = execute_list(mode, cmtable, params, eltlist);
     free_keymanagers();
 
 
