@@ -1,6 +1,6 @@
 /*
  *  loopback-device utilities for cryptmount
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 161 $, $Date: 2007-05-05 22:53:24 +0100 (Sat, 05 May 2007) $
  *  Copyright 2005-2007, RW Penney
  */
 
@@ -188,6 +188,27 @@ int loop_ident(unsigned maj, unsigned min, char *buff, size_t buffsz)
     return !found;
 }
 
+
+int loop_dellist(unsigned devcnt, const dev_t *devids)
+    /* tidy-up list of loopback devices */
+{   unsigned i;
+    char buff[256];
+    int eflag=0;
+
+    if (devids == NULL) return eflag;
+
+    for (i=0; i<devcnt; ++i) {
+        if (loop_ident(major(devids[i]), minor(devids[i]), buff, sizeof(buff))
+          || (loop_destroy(buff) != ERR_NOERROR)) {
+            fprintf(stderr, _("failed to free device (%d,%d)\n"),
+                        major(devids[i]), minor(devids[i]));
+            eflag = 1;
+        }
+    }
+
+    return eflag;
+}
+
 /*
  *  (C)Copyright 2005-2007, RW Penney
  */
