@@ -1,6 +1,6 @@
 /*
  *  declarations for loopback-device utilities for cryptmount
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 161 $, $Date: 2007-05-05 22:53:24 +0100 (Sat, 05 May 2007) $
  *  Copyright 2005-2007, RW Penney
  */
 
@@ -30,15 +30,17 @@
 #define _LOOPUTILS_H
 
 #include <stdlib.h>
+#include <sys/types.h>
 
 int loop_findfree(char *buff, size_t buffsz);
 int loop_setup(const char *dev, const char *file, int flags);
 int loop_ident(unsigned maj, unsigned min, char *buff, size_t buffsz);
 int loop_destroy(const char *dev);
+int loop_dellist(unsigned devcnt, const dev_t *devids);
 
 
 #endif  /* _LOOPUTILS_H */
 
 /*
- *  (C)Copyright 2005-2006, RW Penney
+ *  (C)Copyright 2005-2007, RW Penney
  */
