@@ -1,6 +1,6 @@
 /*
  *  config-table and mount-table utilities for cryptmount
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 161 $, $Date: 2007-05-05 22:53:24 +0100 (Sat, 05 May 2007) $
  *  Copyright 2005-2007, RW Penney
  */
 
@@ -64,6 +64,14 @@ struct statfile     /* information about status file */
 static cment_t *parse_stream(FILE *fp, const char *cfgname);
 
 
+/*  The format for the status-file (version 0) is as follows:
+    # [TITLE]           - one-line text header
+    0                   - integer file-format version number
+    n,[TARGET],uid      - target-name length, target-name, uid of mounter
+    ...                 - further target records
+*/
+
+
 cment_t *alloc_cment(void)
     /* allocate storage for a target-structure */
 {   cment_t *cment;
@@ -426,6 +434,7 @@ tgtstat_t *alloc_tgtstatus(const struct cment *ent)
     ts = (tgtstat_t*)malloc(sizeof(tgtstat_t));
     ts->ident = NULL;
     ts->uid = 0;
+    ts->nx = NULL;
 
     if (ent != NULL) {
         ts->ident = (char*)malloc((size_t)(strlen(ent->ident) + 1));
@@ -437,10 +446,14 @@ tgtstat_t *alloc_tgtstatus(const struct cment *ent)
 
 
 void free_tgtstatus(tgtstat_t *ts)
-    /* free storage of target-status record */
-{
-    if (ts->ident != NULL) free((void*)ts->ident);
-    free((void*)ts);
+    /* free storage of target-status record (or list thereof) */
+{   tgtstat_t *tx=NULL;
+
+    while ((tx = ts) != NULL) {
+        ts = tx->nx;
+        if (tx->ident != NULL) free((void*)tx->ident);
+        free((void*)tx);
+    }
 }
 
 
@@ -480,7 +493,7 @@ tgtstat_t *statfile_read(struct statfile *sf)
     if (fscanf(sf->fp, "%d,", &len) != 1) goto bail_out;
 
     ident = (char*)malloc((size_t)(len + 1));
-    if (fread((void*)ident, (size_t)(len + 1), 1, sf->fp) != 1) {
+    if (fread((void*)ident, (size_t)(len + 1), (size_t)1, sf->fp) != 1) {
         goto bail_out;
     }
     ident[len] = '\0';
@@ -517,6 +530,7 @@ void statfile_close(struct statfile *sf)
 
 
 tgtstat_t *get_tgtstatus(const cment_t *cment)
+    /* find mount/owner status of given target */
 {   char *fname=NULL;
     tgtstat_t *ts=NULL;
     struct statfile *sf;
@@ -543,6 +557,34 @@ tgtstat_t *get_tgtstatus(const cment_t *cment)
 }
 
 
+tgtstat_t *get_all_tgtstatus()
+    /* find list of mount/owner status for all mounted targets */
+{   char *fname=NULL;
+    tgtstat_t *ts=NULL, *head=NULL, **sfp=&head;
+    struct statfile *sf;
+    int badlock;
+
+    (void)cm_path(&fname, "cmstatus");
+    badlock = cm_mutex_lock();
+    sf = statfile_open(fname, "r");
+    if (sf == NULL) goto bail_out;
+
+    while ((ts = statfile_read(sf)) != NULL) {
+        *sfp = ts;
+        sfp = &ts->nx;
+    }
+
+    statfile_close(sf);
+
+  bail_out:
+
+    if (!badlock) cm_mutex_unlock();
+    if (fname != NULL) free((void*)fname);
+
+    return head;
+}
+
+
 int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
 {   char *newfname=NULL,*oldfname=NULL;
     struct statfile *sfin,*sfout;
@@ -550,8 +592,8 @@ int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
     struct stat sbuff;
     int badlock, eflag=0;
 
-    (void)cm_path(&oldfname,"cmstatus");
-    (void)cm_path(&newfname,"cmstatus-temp");
+    (void)cm_path(&oldfname, "cmstatus");
+    (void)cm_path(&newfname, "cmstatus-temp");
     badlock = cm_mutex_lock();
 
     sfout = statfile_open(newfname, "w");
@@ -559,7 +601,7 @@ int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
         eflag = 1; goto bail_out;
     }
 
-    if (stat(oldfname,&sbuff) == 0) {
+    if (stat(oldfname, &sbuff) == 0) {
         sfin = statfile_open(oldfname, "r");
         if (sfin == NULL) {
             statfile_close(sfout);
