@@ -1,6 +1,6 @@
 /*
  *  declarations for config-table & mount-table utilities for cryptmount
- *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  $Revision: 161 $, $Date: 2007-05-05 22:53:24 +0100 (Sat, 05 May 2007) $
  *  Copyright 2005-2007, RW Penney
  */
 
@@ -35,8 +35,10 @@ struct cment;
 
 typedef struct tgtstat
 {
-    char *ident;
-    unsigned long uid;
+    char *ident;            /* unique identifying name of target */
+    unsigned long uid;      /* user-ID responsible for mounting filesystem */
+
+    struct tgtstat *nx;     /* form into linked list */
 } tgtstat_t;
 
 
@@ -50,6 +52,7 @@ void free_config(struct cment **head);
 
 tgtstat_t *alloc_tgtstatus(const struct cment *ent);
 tgtstat_t *get_tgtstatus(const struct cment *ent);
+tgtstat_t *get_all_tgtstatus();
 int put_tgtstatus(const struct cment *ent, const tgtstat_t *tstat);
 void free_tgtstatus(tgtstat_t *tstat);
 
